<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Xss;
use Drupal\iubenda_integration\Exception\InvalidEmbedCodeException;
use Psr\Log\LoggerInterface;

/**
 * Parses embed code snippets related to Iubenda's cookie solution.
 */
final class EmbedCodeParser implements EmbedCodeParserInterface {

  /**
   * Cookie Solution configuration regex.
   *
   * This regex captures the JSON configuration assigned to
   * `_iub.csConfiguration`. It matches the variable assignment and captures
   * the JSON object within curly braces.
   */
  private string $csConfigRegex = '/_iub\.csConfiguration\s*=\s*(\{.*?\});/s';

  /**
   * Cookie Solution language configuration regex.
   *
   * This regex captures the JSON configuration for language settings.
   * It assumes the JSON is well-formed and enclosed within curly braces.
   */
  private string $csLangConfigRegex = '/_iub\.csLangConfiguration\s*=\s*(\{.*?\});/s';

  /**
   * Constructs an EmbedCodeParser object.
   *
   * This class is responsible for parsing embed code snippets related to
   * Iubenda's cookie solution. It extracts the JSON configuration and
   * JavaScript source URLs from the provided embed code snippet.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service for logging messages.
   */
  public function __construct(
    private readonly LoggerInterface $logger,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function decodeCookieSolutionConfig(string $snippet): array {
    $snippet = Xss::filterAdmin($snippet);
    if (preg_match($this->csConfigRegex, $snippet, $csConfigMatch)) {
      $json_config = Json::decode($csConfigMatch[1]);
      if (!is_array($json_config)) {
        throw new InvalidEmbedCodeException('Invalid embed code: csConfiguration is not a valid JSON object.');
      }

      return $json_config;
    }
    else {
      throw new InvalidEmbedCodeException('Invalid embed code: csConfiguration not found.');
    }
  }

  /**
   * Extracts the JSON language configuration from an embed code snippet.
   *
   * This method uses a regular expression to find and extract the JSON
   * configuration assigned to `_iub.csLangConfiguration` within the provided
   * cookie solution embed code snippet.
   *
   * @param string $snippet
   *   The embed code snippet containing the language configuration.
   *
   * @return array
   *   The extracted JSON language configuration as an array. If only one
   *   language is configured, it returns an empty array.
   *
   * @throws \Drupal\iubenda_integration\Exception\InvalidEmbedCodeException
   *   If the embed code does not contain a valid language configuration.
   */
  public function decodeCookieSolutionLangConfig(string $snippet): array {
    $snippet = Xss::filterAdmin($snippet);
    if (preg_match($this->csLangConfigRegex, $snippet, $csLangConfigMatch)) {
      $json_config = Json::decode($csLangConfigMatch[1]);
      if (!is_array($json_config)) {
        throw new InvalidEmbedCodeException('Invalid embed code: csConfiguration is not a valid JSON object.');
      }

      return $json_config;
    }
    else {
      $this->logger->warning('No language configuration found in the embed code snippet, because the Cookie Solution is only for a single language.');

      return [];
    }
  }

}
