<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\iubenda_integration\EmbedCodeParserInterface;
use Drupal\iubenda_integration\Exception\InvalidEmbedCodeException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures the iubenda cookie solution settings.
 */
class CookieSolutionConfigForm extends IubendaConfigFormBase {

  /**
   * The embed code parser service.
   *
   * @var \Drupal\iubenda_integration\EmbedCodeParserInterface
   */
  private EmbedCodeParserInterface $embedCodeParser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    $instance = parent::create($container);

    $parser = $container->get('iubenda_integration.embed_code_parser');
    assert($parser instanceof EmbedCodeParserInterface);
    $instance->embedCodeParser = $parser;

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'iubenda_integration_settings_cookie';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config(self::CONFIG_NAME);

    $form['intro'] = [
      '#type' => 'item',
      '#markup' => $this->t('<b>First</b>, go to your <a href=":url_dash" target="_blank">iubenda Dashboard</a> and click on
        <em>[Your website] > Privacy Controls and Cookie Solution > Edit</em> to activate & configure your Privacy Controls and Cookie Solution.
        <b>Then</b>, paste here your embed code.', [
          ':url_dash' => 'https://www.iubenda.com/en/dashboard',
        ],
      ),
    ];

    $site_id = $this->config(self::CONFIG_NAME)->get('site_id') ?? '';
    $form['cookie_solution_embed_code'] = [
      '#title' => $this->t('Privacy Controls and Cookie Solution embed code'),
      '#description' => $this->t("Where can I @cs_link this code?
        Then, if you need to customise your banner's behaviour and appearance thoroughly,
        @edit_link to configure the cookie banner. When you have done, copy and paste the
        embed code into the above text area.", [
          '@cs_link' => $this->getCookieSolutionDocLink()->toString(),
          '@edit_link' => $this->getIubendaConfigLink($site_id)->toString(),
        ],
      ),
      '#type' => 'textarea',
      '#required' => TRUE,
      '#placeholder' => 'Paste here your embed code',
      '#default_value' => $config->get('cookie_solution_embed_code'),
    ];

    $form['settings'] = [
      '#type' => 'vertical_tabs',
      '#states' => [
        'visible' => [
          ':input[name="cookie_solution_embed_code"]' => ['filled' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $form_element_name = 'cookie_solution_embed_code';
    $embed_code = $form_state->getValue($form_element_name);
    try {
      $decoded_config = $this->embedCodeParser->decodeCookieSolutionConfig($embed_code);
    }
    catch (InvalidEmbedCodeException $e) {
      $this->logger('iubenda_integration')->error($e->getMessage());
      $form_state->setErrorByName($form_element_name, 'The _iub.csConfiguration content is not valid. Please check it and try again.');

      return;
    }

    if (empty($decoded_config['siteId'])) {
      $form_state->setErrorByName($form_element_name, 'The siteId value is missing.');
    }
    if (empty($decoded_config['cookiePolicyId'])) {
      $form_state->setErrorByName($form_element_name, 'The cookiePolicyId value is missing.');
    }

    try {
      $decoded_lang_config = $this->embedCodeParser->decodeCookieSolutionLangConfig($embed_code);
    }
    catch (InvalidEmbedCodeException $e) {
      $this->logger('iubenda_integration')->error($e->getMessage());
      $form_state->setErrorByName($form_element_name, 'The _iub.csLangConfiguration content is not valid. Please check it and try again.');

      return;
    }

    // Here we set the validated values in order to save them later.
    // In this way, we ensure that when we need to use the configuration
    // anywhere in the code, we can retrieve the validated values, not the raw
    // embed code.
    $form_state->setValue('siteId', $decoded_config['siteId']);
    $form_state->setValue('cookiePolicyId', $decoded_config['cookiePolicyId']);
    $form_state->setValue('cookie_solution_config', Json::encode($decoded_config));
    $form_state->setValue('cookie_solution_lang_config', Json::encode($decoded_lang_config));
    if (isset($decoded_config['enableTcf']) && $decoded_config['enableTcf']) {
      $form_state->setValue('enable_tcf', $decoded_config['enableTcf']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    $config = $this->config(self::CONFIG_NAME);
    $values = $form_state->getValues();

    // Extract valid configuration keys from the schema.
    $valid_config_keys = $this->extractValidConfigKeys();
    // Filter form values based on schema definition.
    foreach ($values as $key => $value) {
      if (in_array($key, $valid_config_keys)) {
        $config->set($key, $value);
      }
    }

    $config->save();
  }

}
