<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration\Form;

use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a form for configuring the iubenda Privacy Policy settings.
 */
final class PrivacyPolicyConfigForm extends IubendaConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'iubenda_integration_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['intro'] = [
      '#type' => 'item',
      '#markup' => $this->t('<b>First</b>, go to your @dashboard_link and click on
        <em>[Your website] > Privacy and Cookie Policy > Edit</em> to configure your Privacy Policy.
        <b>Then</b>, paste here the generated Privacy Policy URL.
        <b>Finally</b>, place the @blocks wherever you want to display the Privacy Policy and Cookie Policy on your site.', [
          '@dashboard_link' => $this->getDashboardLink()->toString(),
          '@blocks' => $this->getBlockLayoutLink('Iubenda: Privacy policy')->toString(),
        ],
      ),
    ];

    $form['settings'] = [
      '#type' => 'vertical_tabs',
      '#default_tab' => 'edit-iubenda',
    ];
    $form = $this->buildPrivacyPolicyFormElements($form);
    $form = $this->buildFormElements($form);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config(self::CONFIG_NAME);
    $values = $form_state->getValues();

    // Extract valid configuration keys from the schema.
    $valid_config_keys = $this->extractValidConfigKeys();
    // Filter form values based on schema definition.
    foreach ($values as $key => $value) {
      if (in_array($key, $valid_config_keys)) {
        $config->set($key, $value);
      }
    }

    $config->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Builds the privacy policy form settings.
   *
   * @param array $form
   *   The form array to which the privacy policy settings will be added.
   *
   * @return array
   *   The form with the privacy policy settings.
   */
  private function buildPrivacyPolicyFormElements(array $form): array {
    $config = $this->config(self::CONFIG_NAME);

    $form['iubenda'] = [
      '#type' => 'details',
      '#title' => $this->t('Privacy Policy'),
      '#group' => 'settings',
    ];
    $form['iubenda']['policy_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Privacy Policy URL'),
      '#size' => 60,
      '#pattern' => 'https://www.iubenda.com/privacy-policy/[0-9]+',
      '#description' => $this->t('Insert the Privacy Policy URL from your Iubenda dashboard (e.g. https://www.iubenda.com/privacy-policy/12345678).'),
      '#default_value' => $config->get('policy_url'),
      '#placeholder' => 'https://www.iubenda.com/privacy-policy/12345678',
      '#required' => TRUE,
    ];
    $form['iubenda']['integration'] = [
      '#type' => 'details',
      '#title' => $this->t('Integration'),
      '#open' => TRUE,
      '#summary_attributes' => [
        'class' => ['iubenda-integration-summary'],
      ],
    ];
    $form['iubenda']['integration']['legal_only'] = [
      '#title' => $this->t('Simplified privacy policy'),
      '#description' => $this->t('Enable this option to display a simplified version of the privacy policy, which is suitable for legal compliance.'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('legal_only'),
    ];
    $form['iubenda']['integration']['show_brand'] = [
      '#title' => $this->t('White label'),
      '#description' => $this->t('Enable this option to remove the iubenda brand from the privacy policy button.'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('show_brand'),
    ];
    $form['iubenda']['integration']['style'] = [
      '#title' => $this->t('Link style'),
      '#description' => $this->t('Select the style of the privacy policy link.'),
      '#type' => 'radios',
      '#options' => [
        'iubenda-nostyle' => $this->t('No style'),
        'iubenda-white' => $this->t('White'),
        'iubenda-black' => $this->t('Black'),
      ],
      '#default_value' => $config->get('style'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * Builds the form elements for form validation settings.
   *
   * @param array $form
   *   The form array to which the form validation settings will be added.
   *
   * @return array
   *   The form with the form validation settings.
   */
  private function buildFormElements(array $form): array {
    $config = $this->config('iubenda_integration.settings');

    $form['form_validation'] = [
      '#type' => 'details',
      '#title' => $this->t('Form Validation'),
      '#description' => $this->t('Configure the form validation to ensure users have read and understood the privacy policy before submitting.'),
      '#group' => 'settings',
    ];
    $form['form_validation']['form_element_type'] = [
      '#title' => $this->t('Form element type'),
      '#description' => $this->t('Select the type of form element to use for accepting the privacy policy.'),
      '#type' => 'select',
      '#options' => [
        'checkbox' => $this->t('Checkbox'),
        'radio' => $this->t('Radio'),
      ],
      '#default_value' => $config->get('form_element_type'),
    ];
    $form['form_validation']['form_element_label'] = [
      '#title' => $this->t('Form element label'),
      '#description' => $this->t('This text will be displayed as label of the input form element to accept the privacy policy. "Yes" is the default label.'),
      '#type' => 'textfield',
      '#default_value' => $config->get('form_element_label') ?? 'Yes',
      '#required' => TRUE,
    ];
    $form['form_validation']['forms'] = [
      '#title' => $this->t('Where to Include Privacy Policy Confirmation'),
      '#description' => $this->t('Enter one form ID per line to add the Iubenda Privacy Policy validation element to those forms.'),
      '#type' => 'textarea',
      '#default_value' => $config->get('forms'),
    ];
    $form['form_validation']['link'] = [
      '#title' => $this->t('Privacy Policy link'),
      '#type' => 'fieldset',
    ];
    $form['form_validation']['link']['privacy_policy_pretext'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Link prefix'),
      '#description' => $this->t('Enter the text that will appear immediately before the Privacy Policy link. This text will be shown to users as an introduction to the link.'),
      '#default_value' => $config->get('privacy_policy_pretext'),
    ];
    $form['form_validation']['link']['privacy_policy_text'] = [
      '#title' => $this->t('Link text'),
      '#description' => $this->t('Enter the text that will be used as the clickable Privacy Policy link. This text will be visible to users as the link label.'),
      '#type' => 'textfield',
      '#default_value' => $config->get('privacy_policy_text'),
      '#required' => TRUE,
    ];
    $form['form_validation']['link']['privacy_policy_posttext'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Link suffix'),
      '#description' => $this->t('Enter the text that will appear immediately after the Privacy Policy link. This text will be shown to users as a continuation or explanation following the link.'),
      '#default_value' => $config->get('posttext'),
    ];

    return $form;
  }

}
