<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\GeneratedLink;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\iubenda_integration\Enum\IubendaBlocks;

/**
 * This class contains methods to manage the privacy policy rendering.
 *
 * @group iubenda_integration
 */
class IubendaRenderer implements IubendaRendererInterface {

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $config;

  /**
   * The class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The configuration factory service.
   */
  public function __construct(
    private readonly ConfigFactory $configFactory,
  ) {
    $this->config = $this->configFactory->get('iubenda_integration.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function buildLink(IubendaBlocks $policy, string $link_text = ''): GeneratedLink {
    $url = $this->getPolicyUrl($policy);

    $link_classes = [
      'iubenda-embed',
      'iubenda-noiframe',
    ];

    if (!empty($this->config->get('style'))) {
      $link_classes[] = $this->config->get('style');
    }

    if (!$this->config->get('legal_only') && $policy === IubendaBlocks::IubendaPrivacyPolicy) {
      $link_classes[] = 'iub-legal-only';
    }

    if ($this->config->get('show_brand')) {
      $link_classes[] = 'no-brand';
    }

    if (empty($link_text)) {
      $link_text = $this->config->get('link_text');
    }

    $link_options = [
      'external' => TRUE,
      'attributes' => [
        'class' => $link_classes,
        'title' => trim($link_text),
      ],
    ];
    $iubenda_url = Url::fromUri($url);
    $iubenda_url->setOptions($link_options);

    return Link::fromTextAndUrl($link_text, $iubenda_url)->toString();
  }

  /**
   * {@inheritdoc}
   */
  public function buildString(IubendaBlocks $policy): string {
    $pre_text = $this->config->get('pretext');
    $post_text = $this->config->get('posttext');
    $middle_text = $this->buildLink($policy);

    return trim($pre_text) . ' ' . $middle_text . ' ' . trim($post_text);
  }

  /**
   * {@inheritdoc}
   */
  public function buildFormValidationElement(array &$form, string $form_id): void {
    $form_ids = $this->getFormIds();
    if (empty($form_ids) || !in_array($form_id, $form_ids)) {
      return;
    }

    // Set the privacy checkbox as the last but one submit element.
    $weight = 1;
    $element_types = ['checkbox', 'selectbox', 'item'];
    foreach ($form as $item) {
      if (isset($item['#type']) && in_array($item['#type'], $element_types)) {
        $weight++;
      }
    }

    $form['privacy_policy'] = [
      '#type' => $this->config->get('form_element_type'),
      '#title' => $this->config->get('form_element_label') ?: t('Yes'),
      '#description' => $this->buildString(IubendaBlocks::IubendaPrivacyPolicy),
      '#required' => TRUE,
      '#weight' => $weight,
    ];
  }

  /**
   * Helper function: get a list of form id enabled for Iubenda.
   *
   * @return array
   *   An array of form id enabled for Iubenda privacy policy link.
   */
  private function getFormIds(): array {
    $form_ids_text = $this->config->get('forms');
    $form_ids = ($form_ids_text !== NULL) ? explode("\n", $form_ids_text) : [];

    return array_map('trim', $form_ids);
  }

  /**
   * Helper function: get the URL for the given Iubenda policy.
   *
   * @param \Drupal\iubenda_integration\Enum\IubendaBlocks $policy
   *   The Iubenda policy to get the URL for.
   *
   * @return string
   *   The URL for the given Iubenda policy.
   *
   * @throws \InvalidArgumentException
   *   If an invalid policy type is provided.
   */
  private function getPolicyUrl(IubendaBlocks $policy): string {
    $policy_url = $this->config->get('policy_url') ?? '';
    $terms_and_condition_url = $this->config->get('terms_and_condition_url') ?? '';

    return match($policy->name) {
      IubendaBlocks::IubendaCookiePolicy->name      => "$policy_url/cookie-policy",
      IubendaBlocks::IubendaPrivacyPolicy->name     => $policy_url,
      IubendaBlocks::IubendaTermsAndCondition->name => $terms_and_condition_url,
      default                                       => throw new \InvalidArgumentException('Invalid policy type provided.'),
    };
  }

}
