<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Routing\AdminContext;
use Psr\Log\LoggerInterface;

/**
 * This class contains methods to attach the iubenda library to the page.
 *
 * @group iubenda_integration
 */
class LibraryAttacher implements LibraryAttacherInterface {

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $config;

  /**
   * The class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The configuration factory service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   * @param \Drupal\Core\Routing\AdminContext $adminContext
   *   The admin context service.
   */
  public function __construct(
    private readonly ConfigFactory $configFactory,
    private readonly ModuleHandlerInterface $moduleHandler,
    private readonly LoggerInterface $logger,
    private readonly AdminContext $adminContext,
  ) {
    $this->config = $this->configFactory->get('iubenda_integration.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function attachIubendaLibrary(array &$attachments): void {
    if ($this->isAdminRoute()) {
      // Do not attach the library on admin routes.
      return;
    }
    $privacy_policy_url = $this->config->get('policy_url');
    $terms_and_conditions_url = $this->config->get('terms_and_conditions_url');
    if (empty($privacy_policy_url) && empty($terms_and_conditions_url)) {
      return;
    }
    $attachments['#attached']['library'][] = 'iubenda_integration/iubenda';
  }

  /**
   * {@inheritdoc}
   */
  public function attachCookieSolutionLibrary(array &$attachments): void {
    if ($this->isAdminRoute()) {
      // Do not attach the library on admin routes.
      return;
    }
    $site_id = $this->config->get('siteId');
    $cookie_policy_id = $this->config->get('cookiePolicyId');
    if (empty($site_id) && empty($cookie_policy_id)) {
      $this->logger->warning('Cookie Solution library not attached because siteId or cookiePolicyId is not set in the configuration.');
      return;
    }

    $iubenda_settings = [];
    $cookie_solution_config = Json::decode($this->config->get('cookie_solution_config'));
    if (is_array($cookie_solution_config)) {
      $iubenda_settings['cookieSolutionConfig'] = $cookie_solution_config;
    }
    $cookie_solution_lang_config = Json::decode($this->config->get('cookie_solution_lang_config'));
    if (is_array($cookie_solution_lang_config)) {
      $iubenda_settings['cookieSolutionLangConfig'] = $cookie_solution_lang_config;
    }

    $this->moduleHandler->alter('iubenda_cookie_solution_config', $iubenda_settings);
    $attachments['#attached']['drupalSettings'] = $iubenda_settings;
    $attachments['#attached']['library'][] = 'iubenda_integration/cookie-solution';

    if (isset($cookie_solution_config['enableTcf']) && $cookie_solution_config['enableTcf']) {
      $attachments['#attached']['library'][] = 'iubenda_integration/cookie-solution-tcf';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function attachConsentSolutionLibrary(array &$attachments): void {
    if ($this->isAdminRoute() || empty($this->config->get('api_key'))) {
      // Do not attach the library on admin routes or if the API key is not set.
      return;
    }
    /*
     * @todo Avoid to attach the API key in the JS settings, it is insecure.
     *
     * $attachments['#attached']['drupalSettings']['iubendaIntegration']
     * ['consentSolution'] = [
     *   'api_key' => $this->config->get('api_key'),
     * ];
     */
    $attachments['#attached']['library'][] = 'iubenda_integration/consent-solution';
  }

  /**
   * Checks if the current route is an admin route.
   *
   * @return bool
   *   TRUE if the current route is an admin route, FALSE otherwise.
   */
  private function isAdminRoute(): bool {
    return $this->adminContext->isAdminRoute();
  }

}
