<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration\Plugin\Block;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\iubenda_integration\Enum\IubendaBlocks;
use Drupal\iubenda_integration\IubendaRendererInterface;
use Drupal\iubenda_integration\Plugin\Derivative\IubendaBlockDeriver as IubendaBlockDeriver;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block for displaying iubenda privacy policy links.
 *
 * This block allows site administrators to add iubenda privacy policy links
 * with customizable text before and after the link.
 */
#[Block(
  id: "iubenda_integration_block",
  admin_label: new TranslatableMarkup("iubenda block"),
  category: new TranslatableMarkup("iubenda"),
  deriver: IubendaBlockDeriver::class,
)]
final class IubendaBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    private readonly IubendaRendererInterface $iubendaRenderer,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $iubenda_renderer = $container->get('iubenda_integration.renderer');
    assert($iubenda_renderer instanceof IubendaRendererInterface);

    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $iubenda_renderer,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    $derivative_id = $this->getDerivativeId();

    return [
      'iubenda_integration_block' =>
        [
          'text_prefix' => '',
          'text' => IubendaBlocks::tryFromName($derivative_id)?->value ?? '',
          'text_suffix' => '',
        ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form = parent::blockForm($form, $form_state);
    $configurations_iubenda = $this->configuration['iubenda_integration_block'];

    $form['iubenda_integration_block'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Iubenda block settings'),
    ];
    $form['iubenda_integration_block']['text_prefix'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Link prefix'),
      '#description' => $this->t('Insert the text before the link.'),
      '#default_value' => $configurations_iubenda['text_prefix'],
    ];
    $form['iubenda_integration_block']['text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link text'),
      '#description' => $this->t('Insert text that will be displayed as the link.'),
      '#default_value' => $configurations_iubenda['text'],
      '#required' => TRUE,
    ];
    $form['iubenda_integration_block']['text_suffix'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Link suffix'),
      '#description' => $this->t('Insert the text after the link.'),
      '#default_value' => $configurations_iubenda['text_suffix'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $configurations = $form_state->getValue('iubenda_integration_block');
    $this->configuration['iubenda_integration_block'] = $configurations;
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $derivative_id = $this->getDerivativeId();

    return [
      '#theme' => 'block__iubenda_privacy_policy',
      '#pre_text' => Xss::filterAdmin($this->configuration['iubenda_integration_block']['text_prefix']),
      '#link' => $this->iubendaRenderer->buildLink(
        IubendaBlocks::tryFromName($derivative_id),
        $this->configuration['iubenda_integration_block']['text'],
      ),
      '#post_text' => Xss::filterAdmin($this->configuration['iubenda_integration_block']['text_suffix']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    return Cache::mergeTags(
      parent::getCacheTags(),
      ['iubenda_integration.settings'],
    );
  }

}
