<?php

declare(strict_types=1);

namespace Drupal\iubenda_integration\Traits;

use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;

/**
 * Provides a trait for generating iubenda links.
 *
 * @package Drupal\iubenda_integration\Traits
 */
trait IubendaLinkTrait {

  use StringTranslationTrait;

  /**
   * Builds the iubenda dashboard link.
   *
   * @param string $text
   *   The text to display for the link.
   *
   * @return \Drupal\Core\Link
   *   iubenda dashboard link as HTML markup.
   */
  protected function getDashboardLink(string $text = 'iubenda Dashboard'): Link {
    return Link::fromTextAndUrl(
      // Prevent 'Only string literals should be passed to t() where possible'.
      // phpcs:ignore
      new TranslatableMarkup($text),
      Url::fromUri(
        'https://www.iubenda.com/en/dashboard',
        ['attributes' => ['target' => '_blank']],
      ),
    );
  }

  /**
   * Builds the iubenda block layout link.
   *
   * @param string $text
   *   The text to display for the link.
   *
   * @return \Drupal\Core\Link
   *   iubenda block layout link as HTML markup.
   */
  protected function getBlockLayoutLink(string $text = 'iubenda'): Link {
    return Link::fromTextAndUrl(
      // Prevent 'Only string literals should be passed to t() where possible'.
      // phpcs:ignore
      new TranslatableMarkup("$text block"),
      Url::fromRoute(
        'block.admin_display',
        [],
        ['attributes' => ['target' => '_blank']],
      ),
    );
  }

  /**
   * Builds the iubenda Cookie Solution documentation link.
   *
   * @param string $text
   *   The text to display for the link.
   *
   * @return \Drupal\Core\Link
   *   Link to the iubenda Cookie Solution documentation.
   */
  protected function getCookieSolutionDocLink(string $text = 'find'): Link {
    return Link::fromTextAndUrl(
      // Prevent 'Only string literals should be passed to t() where possible'.
      // phpcs:ignore
      new TranslatableMarkup($text),
      Url::fromUri(
        'https://www.iubenda.com/en/help/1177',
        ['attributes' => ['target' => '_blank']],
      ),
    );
  }

  /**
   * Builds the iubenda Consent Solution documentation link.
   *
   * @param string $text
   *   The text to display for the link.
   *
   * @return \Drupal\Core\Link
   *   Link to the iubenda Consent Solution documentation.
   */
  protected function getConsentSolutionDocLink(string $text = 'find'): Link {
    return Link::fromTextAndUrl(
      // Prevent 'Only string literals should be passed to t() where possible'.
      // phpcs:ignore
      new TranslatableMarkup($text),
      Url::fromUri(
        'https://www.iubenda.com/en/help/6473-consent-solution-js-documentation#generate-embed',
        ['attributes' => ['target' => '_blank']],
      ),
    );
  }

  /**
   * Builds the iubenda site configuration link.
   *
   * @param string $site_id
   *   Your site ID.
   * @param string $text
   *   The text to display for the link.
   *
   * @return \Drupal\Core\Link
   *   Link to the iubenda site configuration page.
   */
  protected function getIubendaConfigLink(string $site_id, string $text = 'click here'): Link {
    return Link::fromTextAndUrl(
      // Prevent 'Only string literals should be passed to t() where possible'.
      // phpcs:ignore
      new TranslatableMarkup($text),
      Url::fromUri(
        "https://www.iubenda.com/en/flow/$site_id",
        ['attributes' => ['target' => '_blank']],
      ),
    );
  }

}
