<?php

declare(strict_types=1);

namespace Drupal\Tests\iubenda_integration\Kernel;

use Drupal\iubenda_integration\EmbedCodeParserInterface;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the iubenda embed code parser.
 *
 * @group iubenda_integration
 */
class EmbedCodeParserTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'iubenda_integration',
  ];

  /**
   * Test the iubenda Cookie Solution snippet.
   *
   * @param string $snippet_filename
   *   The name of the file containing the embed code snippet.
   *
   * @dataProvider providerTestCookieSolutionSnippet
   *
   * @throws \Drupal\iubenda_integration\Exception\InvalidEmbedCodeException
   */
  public function testCookieSolutionSnippet(string $snippet_filename): void {
    $snippet = $this->loadEmbedCodeSnippet($snippet_filename);
    $parser = \Drupal::service('iubenda_integration.embed_code_parser');
    $this->assertInstanceOf(EmbedCodeParserInterface::class, $parser);
    $config = $parser->decodeCookieSolutionConfig($snippet);
    $this->assertIsArray($config, 'The decoded configuration should be an array.');
    $this->assertArrayHasKey('siteId', $config, 'The configuration should contain the siteId key.');
    $this->assertIsNumeric($config['siteId'], 'The decoded configuration should contain the siteId key.');
    $this->assertArrayHasKey('cookiePolicyId', $config, 'The configuration should contain the cookiePolicyId key.');
    $this->assertIsNumeric($config['cookiePolicyId'], 'The decoded configuration should contain the cookiePolicyId key.');
    $this->assertArrayHasKey('banner', $config, 'The configuration should contain the banner key.');
    $this->assertIsArray($config['banner'], 'The banner key should contain an array.');

    if ($snippet_filename === 'cookiesolutionsnippet-tcf.txt') {
      // For the TCF snippet, we expect the enableTcf to be present.
      $this->assertArrayHasKey('enableTcf', $config, 'The configuration should contain the enableTcf key.');
      $this->assertTrue($config['enableTcf'], 'The enableTcf key should contain true.');
      $this->assertIsArray($config['tcfPurposes'], 'The tcfPurposes key should contain an array.');
    }
    else {
      // For the non-TCF snippet, we do not expect the enableTcf key.
      $this->assertArrayNotHasKey('enableTcf', $config, 'The configuration should not contain the enableTcf key.');
    }
  }

  /**
   * Data provider for the testCookieSolutionSnippet method.
   *
   * @return array[]
   *   The data provider for the testCookieSolutionSnippet method.
   */
  public function providerTestCookieSolutionSnippet(): array {
    return [
      [
        'cookiesolutionsnippet.txt',
        'cookiesolutionsnippet-tcf.txt',
      ],
    ];
  }

  /**
   * Loads an embed code snippet from a file.
   *
   * @param string $filename
   *   The name of the file containing the embed code snippet.
   *
   * @return string
   *   The contents of the file asa string.
   *
   * @throws \RuntimeException
   */
  private function loadEmbedCodeSnippet(string $filename): string {
    $file_path = __DIR__ . '/../../assets/' . $filename;
    if (!file_exists($file_path)) {
      throw new \RuntimeException(sprintf('File %s does not exist.', $file_path));
    }
    return file_get_contents($file_path);
  }

}
